@@ -74,6 +74,11 @@ webkit_editing_behavior_get_type(void);
 WEBKIT_API GType
 webkit_navigation_response_get_type(void);
 
+#define WEBKIT_TYPE_CACHE_MODEL webkit_cache_model_get_type()
+
+WEBKIT_API GType
+webkit_cache_model_get_type(void);
+
 #define WEBKIT_TYPE_WEB_VIEW_TARGET_INFO webkit_web_view_target_info_get_type()
 
 WEBKIT_API GType
