@@ -82,6 +82,9 @@ webkit_web_history_item_get_original_uri      (WebKitWebHistoryItem *web_history
 WEBKIT_API gdouble
 webkit_web_history_item_get_last_visited_time (WebKitWebHistoryItem *web_history_item);
 
+WEBKIT_API WebKitWebHistoryItem*
+webkit_web_history_item_copy                  (WebKitWebHistoryItem *web_history_item);
+
 G_END_DECLS
 
 #endif /* webkitwebhistoryitem_h */
